package core

import (
	"git.urbach.dev/cli/q/src/ast"
	"git.urbach.dev/cli/q/src/cpu"
	"git.urbach.dev/cli/q/src/expression"
	"git.urbach.dev/cli/q/src/token"
)

// Execute executes an operation on a register with a value operand.
func (f *Function) Execute(operation token.Token, register cpu.Register, expr *expression.Expression) error {
	if expr.IsLeaf() {
		return f.ExecuteToken(operation, register, expr.Token)
	}

	if ast.IsFunctionCall(expr) {
		_, err := f.CompileCall(expr)

		if err != nil {
			return err
		}

		return f.ExecuteRegisterRegister(operation, register, f.CPU.Output[0])
	}

	tmp := f.NewRegister()
	defer f.FreeRegister(tmp)

	_, err := f.ExpressionToRegister(expr, tmp)

	if err != nil {
		return err
	}

	return f.ExecuteRegisterRegister(operation, register, tmp)
}
